import { BehaviorSubject as BehaviorSubject$1 } from 'rxjs/BehaviorSubject';
import { AsyncSubject as AsyncSubject$1 } from 'rxjs/AsyncSubject';
import { Inject, InjectionToken, NgModule, Pipe } from '@angular/core';
import { CommonModule } from '@angular/common';
import { createStore } from 'redux/index';

class ReduxRegistryReducerItem {
}
class ReduxRegistry {
    /**
     * @param {?} store
     * @return {?}
     */
    static registerStore(store) {
        ReduxRegistry._store.next(store);
        ReduxRegistry._store.complete();
    }
    /**
     * @param {?} stateName
     * @param {?} type
     * @param {?} reducer
     * @return {?}
     */
    static registerReducer(stateName, type, reducer) {
        ReduxRegistry._reducers[type] = ReduxRegistry._reducers[type] || [];
        ReduxRegistry._reducers[type].push({ stateName, reducer });
    }
    /**
     * @param {?} stateName
     * @param {?} module
     * @return {?}
     */
    static registerModule(stateName, module) {
        ReduxRegistry.getStore().then((store) => {
            let /** @type {?} */ initialState = {};
            if (module.onReduxInit) {
                initialState = module.onReduxInit() || {};
            }
            store.dispatch({
                payload: {
                    initialState,
                    stateName,
                },
                type: ReduxRegistry.ACTION_REGISTER_MODULE,
            });
        });
    }
    /**
     * @param {?} type
     * @return {?}
     */
    static getReducerItemsByType(type) {
        return ReduxRegistry._reducers[type] || [];
    }
    /**
     * @return {?}
     */
    static getStore() {
        return new Promise(ReduxRegistry._store.subscribe.bind(ReduxRegistry._store));
    }
}
ReduxRegistry.ACTION_REGISTER_MODULE = `${ReduxRegistry.name}://registerModule`;
ReduxRegistry._store = new AsyncSubject$1();
ReduxRegistry._reducers = {};

/**
 * @template S
 * @param {?} state
 * @param {?} selector
 * @return {?}
 */
function selectByState(state, selector) {
    const /** @type {?} */ selectorArray = selector.split('/');
    if (selectorArray[0] !== '') {
        throw new Error('Relative selectors are not supported yet!');
    }
    return selectorArray
        .filter((propertyKey) => propertyKey !== '')
        .reduce((previousValue, propertyKey) => {
        if (!previousValue || !previousValue[propertyKey]) {
            return null;
        }
        return previousValue[propertyKey];
    }, state);
}
/**
 * @template S
 * @param {?} selector
 * @return {?}
 */
function select(selector) {
    const /** @type {?} */ subject = new BehaviorSubject$1(null);
    ReduxRegistry.getStore().then(store => {
        subject.next(selectByState(store.getState(), selector));
        store.subscribe(() => subject.next(selectByState(store.getState(), selector)));
    });
    return subject.asObservable();
}

/**
 * @param {?} action
 * @return {?}
 */
function dispatch(action) {
    ReduxRegistry.getStore().then(store => store.dispatch(action));
}
/**
 * @param {?=} customType
 * @return {?}
 */
function ReduxAction(customType) {
    return (target, propertyKey, descriptor) => {
        const /** @type {?} */ method = descriptor.value;
        const /** @type {?} */ type = customType || [/** @type {?} */ (target['name']) || target.constructor.name,
            propertyKey,
        ].join(target['name'] ? '/static/' : '/');
        const /** @type {?} */ proxyFunction = function () {
            const /** @type {?} */ payload = method.apply(this, arguments);
            if (payload instanceof Promise) {
                payload.then((p) => dispatch({ payload: p, type }));
            }
            else {
                dispatch({ payload, type });
            }
            return payload;
        };
        proxyFunction['__@ReduxAction'] = { type };
        descriptor.value = proxyFunction;
    };
}

/**
 * @template S, P
 * @param {?} typeArray
 * @return {?}
 */
function ReduxReducer(typeArray) {
    const /** @type {?} */ types = (Array.isArray(typeArray) ? typeArray : [typeArray]).map((t) => {
        if (typeof t === 'function') {
            if (!t['__@ReduxAction'] || !t['__@ReduxAction'].type) {
                throw new Error('Unable to resolve action type. Make sure you\'ve decorated the target by @ReduxAction');
            }
            return t['__@ReduxAction'].type;
        }
        return t;
    });
    return (target, propertyKey) => {
        if (typeof target === 'object') {
            target[propertyKey].bind(target);
        }
        if (!target['__@ReduxReducer']) {
            target['__@ReduxReducer'] = [];
        }
        types.forEach(type => {
            target['__@ReduxReducer'].push({
                actionType: type,
                reducerFunction: target[propertyKey].bind(target),
            });
        });
        return target[propertyKey];
    };
}

/**
 * @param {?=} config
 * @return {?}
 */
function Redux(config) {
    return (constructor) => {
        return class extends constructor {
            constructor() {
                super(...arguments);
                this.redux = (() => {
                    const /** @type {?} */ mergedConfig = Object.assign({
                        reducers: [],
                        stateName: constructor.name,
                    }, config || {});
                    const { stateName, reducers } = mergedConfig;
                    // constructor[ '__@Redux' ] = mergedConfig;
                    ReduxRegistry.registerModule(stateName, this);
                    reducers.reduce((previousValue, currentValue) => {
                        return previousValue
                            .concat(currentValue['__@ReduxReducer'] || [])
                            .concat(currentValue['prototype']['__@ReduxReducer'] || []);
                    }, []).forEach((r) => ReduxRegistry.registerReducer(stateName, r.actionType, r.reducerFunction));
                    return config;
                })();
            }
        };
    };
}

/**
 * @param {?} selector
 * @return {?}
 */
function ReduxSelect(selector) {
    return (target, propertyKey) => {
        Object.defineProperty(target, propertyKey, {
            configurable: true,
            enumerable: true,
            value: select(selector),
        });
    };
}

class ReduxSelectPipe {
    /**
     * @param {?} selector
     * @return {?}
     */
    transform(selector) {
        return select(selector);
    }
}
ReduxSelectPipe.decorators = [
    { type: Pipe, args: [{
                name: 'reduxSelect',
            },] },
];
/**
 * @nocollapse
 */
ReduxSelectPipe.ctorParameters = () => [];

const rootReducer = (state, action) => {
    if (action.type === ReduxRegistry.ACTION_REGISTER_MODULE) {
        const { stateName, initialState } = action.payload;
        return Object.assign({}, state, {
            [stateName]: initialState,
        });
    }
    return ReduxRegistry.getReducerItemsByType(action.type)
        .reduce((stateToReduce, reducerItem) => {
        const { stateName, reducer } = reducerItem;
        return Object.assign({}, stateToReduce, {
            [stateName]: reducer(stateToReduce[stateName], action),
        });
    }, state);
};

const REDUX_MODULE_CONFIG = new InjectionToken('REDUX_MODULE_CONFIG');
const IS_ROOT_MODULE = new InjectionToken('IS_ROOT_MODULE');
class ReduxModule {
    /**
     * @param {?=} isRootModule
     * @param {?=} config
     */
    constructor(isRootModule = false, config = null) {
        if (isRootModule) {
            config = Object.assign({
                store: null,
            }, config || {});
            ReduxRegistry.registerStore(config.store || createStore(rootReducer, {}));
        }
    }
    /**
     * @return {?}
     */
    static forChild() {
        return {
            ngModule: ReduxModule,
            providers: [
                { provide: IS_ROOT_MODULE, useValue: false },
            ],
        };
    }
    /**
     * @param {?=} config
     * @return {?}
     */
    static forRoot(config) {
        return {
            ngModule: ReduxModule,
            providers: [
                { provide: REDUX_MODULE_CONFIG, useValue: config },
                { provide: IS_ROOT_MODULE, useValue: true },
            ],
        };
    }
}
ReduxModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    ReduxSelectPipe,
                ],
                exports: [
                    ReduxSelectPipe,
                ],
                imports: [
                    CommonModule,
                ],
                providers: [],
            },] },
];
/**
 * @nocollapse
 */
ReduxModule.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [IS_ROOT_MODULE,] },] },
    { type: undefined, decorators: [{ type: Inject, args: [REDUX_MODULE_CONFIG,] },] },
];

/**
 * Generated bundle index. Do not edit.
 */

export { selectByState, select, ReduxAction, ReduxReducer, Redux, ReduxSelect, ReduxSelectPipe, REDUX_MODULE_CONFIG, IS_ROOT_MODULE, ReduxModule, ReduxRegistryReducerItem, ReduxRegistry, rootReducer };
//# sourceMappingURL=ngx-redux-core.js.map
